@@ -27,21 +27,27 @@ SOFTWARE.
 #pragma once
 
 #include <windows.h>
+#include <objbase.h>
 #include <npapi.h>
+#if (_WIN32_WINNT < 0x0501)
+#undef _WIN32_WINNT
+#define _WIN32_WINNT 0x0501
+#endif
 #include <ntsecapi.h>
+#include <tchar.h>
 #include <strsafe.h>
 
 
-#define REG_CLIENT_PARMS_KEY            "SYSTEM\\CurrentControlSet\\Services\\TransarcAFSDaemon\\Parameters"
-#define REG_CLIENT_PROVIDER_KEY			"SYSTEM\\CurrentControlSet\\Services\\TransarcAFSDaemon\\NetworkProvider"
-#define REG_CLIENT_DOMAINS_SUBKEY		"Domain"
+#define REG_CLIENT_DOMAINS_SUBKEY	"Domain"
 #define REG_CLIENT_RETRY_INTERVAL_PARM  "LoginRetryInterval"
 #define REG_CLIENT_SLEEP_INTERVAL_PARM	"LoginSleepInterval"
 #define REG_CLIENT_FAIL_SILENTLY_PARM   "FailLoginsSilently"
 #define REG_CLIENT_TRACE_OPTION_PARM	"TraceOption"
 #define REG_CLIENT_LOGON_OPTION_PARM	"LogonOptions"
 #define REG_CLIENT_LOGON_SCRIPT_PARMW	L"LogonScript"
-#define DEFAULT_RETRY_INTERVAL          30                        /* seconds*/
+#define REG_CLIENT_THESE_CELLS_PARM     "TheseCells"
+#define REG_CLIENT_LOGOFF_TOKENS_PARM	"LogoffPreserveTokens"
+#define DEFAULT_RETRY_INTERVAL          60                        /* seconds*/
 #define DEFAULT_FAIL_SILENTLY           FALSE
 #define DEFAULT_SLEEP_INTERVAL          5                         /* seconds*/
 #define DEFAULT_LOGON_OPTION			1
@@ -64,11 +70,12 @@ extern DWORD TraceOption;
 typedef struct LogonOptions_type {
 	DWORD	LogonOption;
 	BOOLEAN	failSilently;
-	int		retryInterval;
-	int		sleepInterval;
+	int	retryInterval;
+	int	sleepInterval;
 	char *	smbName;
 	LPWSTR	logonScript;
 	DWORD	flags; /* LOGON_FLAG_* */
+        char *  theseCells;
 } LogonOptions_t;
 
 /* */
