@@ -13,7 +13,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/viced/host.c,v 1.93.2.12 2006/09/05 22:05:11 shadow Exp $");
+    ("$Header: /cvs/openafs/src/viced/host.c,v 1.93.2.14 2007/01/05 23:06:56 shadow Exp $");
 
 #include <stdio.h>
 #include <errno.h>
@@ -1122,8 +1122,11 @@ h_Enumerate_r(int (*proc) (), struct host *enumstart, char *param)
 	held = (*proc) (host, held, param);
 	if (!H_ENUMERATE_ISSET_HELD(held))
 	    h_Release_r(host); /* this might free up the host */
-	if (H_ENUMERATE_ISSET_BAIL(held))
+	if (H_ENUMERATE_ISSET_BAIL(held)) {
+	    if (!H_ENUMERATE_ISSET_HELD(nheld))
+		h_Release_r(next); /* this might free up the host */
 	    break;
+	}
     }
 }				/*h_Enumerate_r */
 
@@ -1694,7 +1697,7 @@ h_InitHostPackage()
 		break;
 	}
 
-	if (i=0) {
+	if (i == 0) {
 	    ViceLog(0,
 		    ("afs_krb_get_lrealm failed, using %s.\n",
 		     localcellname));
